% DZS
% fuzzy hotely
% patrik dudits

le(X,Y) :- X < Y; X = Y.
ge(X,Y) :- X > Y; X = Y.

% fuzzy gate
% 1 |           _,------
%   |        _,'  |
%   |     _,'     |
% 0 +=====---------------
%         T1      T2
gate(X, T1,  _, 0) :- le(X,T1),!.
gate(X,  _, T2, 1) :- ge(X,T2),!.
gate(X, T1, T2, Y) :- ge(X,T1), le(X,T2), A is (X-T1), B is (T2-T1), Y is A/B.

% invgate
invgate(X, T1, T2, Y) :- gate(X, T1, T2, Z), Y is 1-Z.

% ramp          
% 1 |           _,------,_
%   |        _,'  |    |  '-,_
%   |     _,'     |    |      '-,_
% 0 +=====------------------------======
%         T1      T2   T3         T4
ramp(X, T1, _,  _, T4, 0) :- le(X,T1) ; ge(X,T4),!.
ramp(X,  _, T2, T3, _, 1) :- ge(X,T2), le(X,T3),!.
ramp(X, T1, T2,  _,  _, Y) :- ge(X,T1), le(X,T2), gate(X, T1, T2, Y),!.
ramp(X,  _,  _, T3, T4, Y) :- ge(X,T3), le(X,T4), invgate(X, T3, T4, Y),!.

% blizko
% povedzme, ze 2km je uz daleko

blizko(X, Y):-invgate(X,0,2,Y).

% lacny
% pod 120Sk, to uz asi nebude velmi pohodlne,
% nad 800 je to uz drahe (teda v zavislosti od toho kde)

lacny(X, Y):-ramp(X,120, 180, 800, 1100,Y).

% novy
% budova je stara (na byvanie) po 30 rokoch
% skutocne novy hotel ma podla mna menej ako 5 rokov

novy(X,Y):-invgate(X,5,30,Y).

% agregacia
% 4*lacny+3*blizko+novy/8

agr(X,Y):-hotel(X,Vzd,Cena,Vek),blizko(Vzd,B), lacny(Cena,L), novy(Vek, N), Y is (4*L+3*B+N)/8.

% hotely
hotel(slovan, 1.5, 1000, 20).
hotel(palace, 1.5, 800, 2).
hotel(medicka, 1, 130, 25).
hotel(alessandria, 1200, 0.8, 2).
hotel(ambassador, 900, 0.4, 2).


find(Z) :- findall(A-[Meno, Vzd, Cena, Vek], (hotel(Meno, Vzd, Cena, Vek), agr(Meno, A)), X), keysort(X, Y), reverse(Y, Z).
